﻿using System.Collections.Generic;
using System.Diagnostics;
using System.Threading;
using AreaCalculationDomain;
using Cylinders;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace AreaCalculation {
  [TestClass]
  public class UnitTests {

    [TestMethod, TestCategory("C#")]
    public void CalculatingSinglePsiWithCSharpGivesSameResultsAsCalculatingWithSqlServer() {
      // actual values: WA=4.005	WB=4.010	H=8.000	Weight=79060	 4x4x8   PSI result:	6270
      var cyl = CreateCylinder(1, 4.005, 4.010, 8.000, 79060, CylinderType.FourFourEightCylinder);
      CylinderCalculator.UpdateCylinders(new List<CylinderMeasurement> { cyl });
      Assert.AreEqual(6270, cyl.Psi);
    }

    [TestMethod, TestCategory("C#"), TestCategory("Set")]
    public void CalculatingSetPsiWithCSharpGivesSameResultsAsCalculatingWithSqlServer() {
      // 8.510	4.005	4.010	8.000	79060	 4x4x8  	6270
      // 8.490	4.008	4.006	8.000	82630	 4x4x8  	6550
      //28.040	5.967	5.984	12.000	137970	6x6x12 4920
      // 7.110	3.636	3.722	6.891	57670	 3x3x6   	4170
      // 0.600	1.982	2.022	2.000	8540	2x2x2     	2090
      var cylList = new List<CylWithActualPsi>();
      cylList.Add(new CylWithActualPsi(CreateCylinder(1, 4.005, 4.010, 8.000, 79060, CylinderType.FourFourEightCylinder), 6270));
      cylList.Add(new CylWithActualPsi(CreateCylinder(2, 4.008, 4.006, 8.000, 82630, CylinderType.FourFourEightCylinder), 6550));
      cylList.Add(new CylWithActualPsi(CreateCylinder(3, 5.967, 5.984, 12, 137970, CylinderType.SixSixTwelveCylinder), 4920));
      cylList.Add(new CylWithActualPsi(CreateCylinder(4, 3.636, 3.722, 6.891, 57670, CylinderType.ThreeThreeSixCylinder), 4170));
      cylList.Add(new CylWithActualPsi(CreateCylinder(5, 1.982, 2.022, 2.000, 8540, CylinderType.TwoTwoTwoCylinder), 2090));
      cylList.Add(new CylWithActualPsi(CreateCylinder(6, 6.036, 5.975, 12.000, 100370, CylinderType.SixSixTwelveCylinder), 3540));
      cylList.Add(new CylWithActualPsi(CreateCylinder(7, 3.999, 3.998, 8.000, 71830, CylinderType.FourFourEightCylinder), 5720));

      var cyls = new List<CylinderMeasurement>();
      foreach (var cylsWithPsi in cylList) {
        cyls.Add(cylsWithPsi.Cyl);
      }

      CylinderCalculator.UpdateCylinders(cyls);

      foreach (var cyl in cyls) {
        Assert.AreEqual(cylList.Find(c => c.Cyl.Id == cyl.Id).Psi, cyl.Psi);
      }

    }


    private CylinderMeasurement CreateCylinder(int fakeId, double wA, double wB, double h, int load, CylinderType cylType) {
      var cyl = new CylinderMeasurement(wA, wB, h);
      cyl.UpdateLoadPoundsAndTypeEnum(load, cylType);
      //fake the id, need it for the assert comparison
      cyl.GetType().GetProperty("Id").SetValue(cyl, fakeId);
      return cyl;
    }

    [TestMethod, TestCategory("F#")]
    public void CalculatingSinglePsiWithFSharpGivesSameResultsAsCalculatingWithSqlServer() {

      var cyl = CreateCylinder(4, 3.636, 3.722, 6.891, 57670, CylinderType.ThreeThreeSixCylinder);
      var newcyls = Cylinders.calcPsi.getPsi(new[] { cyl });
      Assert.AreEqual(4170, newcyls[0].Psi);
    }

    [TestMethod, TestCategory("F#"), TestCategory("Set")]
    public void CalculatingSetPsiWithFSharpGivesSameResultsAsCalculatingWithSqlServer() {
      var cylList = new List<CylWithActualPsi>();
      cylList.Add(new CylWithActualPsi(CreateCylinder(1, 4.005, 4.010, 8.000, 79060, CylinderType.FourFourEightCylinder), 6270));
      cylList.Add(new CylWithActualPsi(CreateCylinder(2, 4.008, 4.006, 8.000, 82630, CylinderType.FourFourEightCylinder), 6550));
      cylList.Add(new CylWithActualPsi(CreateCylinder(3, 5.967, 5.984, 12, 137970, CylinderType.SixSixTwelveCylinder), 4920));
      cylList.Add(new CylWithActualPsi(CreateCylinder(4, 3.636, 3.722, 6.891, 57670, CylinderType.ThreeThreeSixCylinder), 4170));
      cylList.Add(new CylWithActualPsi(CreateCylinder(5, 1.982, 2.022, 2.000, 8540, CylinderType.TwoTwoTwoCylinder), 2090));
      cylList.Add(new CylWithActualPsi(CreateCylinder(6, 6.036, 5.975, 12.000, 100370, CylinderType.SixSixTwelveCylinder), 3540));
      cylList.Add(new CylWithActualPsi(CreateCylinder(7, 3.999, 3.998, 8.000, 71830, CylinderType.FourFourEightCylinder), 5720));

      var cyls = new List<CylinderMeasurement>();
      foreach (var cylsWithPsi in cylList) {
        cyls.Add(cylsWithPsi.Cyl);
      }

      var newcyls = calcPsi.getPsi(cyls.ToArray());
      foreach (var cyl in newcyls) {
        Assert.AreEqual(cylList.Find(c => c.Cyl.Id == cyl.Id).Psi, cyl.Psi);

      }
    }
  }

  public struct CylWithActualPsi {
    public CylWithActualPsi(CylinderMeasurement cylinderMeasurement, int psi) {
      Cyl = cylinderMeasurement;
      Psi = psi;
    }
    public CylinderMeasurement Cyl;
    public int Psi;
  }
}